import type {
  AxiosError,
  AxiosInstance,
  AxiosRequestConfig,
  AxiosResponse,
  InternalAxiosRequestConfig
} from 'axios'
import axios, { HttpStatusCode } from 'axios'
import qs from 'qs'
import type { ResultData } from '@/typings/api'
import { useAuthStore } from '@/stores/modules/auth'
import { showFailToast, showConfirmDialog } from 'vant'

/**
 * axios配置
 */
const config = {
  baseURL: import.meta.env.VITE_API_URL,
  timeout: 8000,
  paramsSerializer: {
    serialize(params: Recordable) {
      for (const key in params) {
        if (params[key] === '' || params[key] === null) {
          delete params[key]
        }
      }
      return qs.stringify(params, { allowDots: true })
    }
  }
}

class RequestHttp {
  service: AxiosInstance

  /**
   * 请求构造函数
   * @param config
   */
  public constructor(config: AxiosRequestConfig) {
    // 创建axios实例
    this.service = axios.create(config)
    // 请求拦截
    this.service.interceptors.request.use(
      (config: InternalAxiosRequestConfig) => {
        // 请求头携带token
        const { tokenGet } = useAuthStore()
        if (tokenGet) {
          config.headers['Authorization'] = tokenGet
        }
        return config
      },
      (error: AxiosError) => {
        // 请求错误处理
        return Promise.reject(error)
      }
    )
    // 响应拦截
    this.service.interceptors.response.use(
      (response: AxiosResponse) => {
        const { data } = response
        // 响应拦截处理
        return data
      },
      (error: AxiosError<ResultData<any>>) => {
        const authStore = useAuthStore()
        const { response, message } = error
        const data = response?.data
        const errMsg = data?.message || message
        // 响应错误处理
        if (response?.status === HttpStatusCode.Unauthorized) {
          if (window.location.pathname.toLocaleLowerCase() !== '/login') {
            authStore.$reset()
            window.location.reload()
          }
        }else if (response?.status === HttpStatusCode.Forbidden) {
          showConfirmDialog({
            message: '该资源没有访问权限,是否重新登录？'
          }).then(() => {
            authStore.logout()
          })
        } else {
          showFailToast(errMsg || '请求失败')
        }
        // 判断客户端是否联网
        if (!window.navigator.onLine) {
          showFailToast('网络异常')
        }
        return Promise.reject(response?.data || error)
      }
    )
  }

  /**
   * get请求
   * @param url 请求地址
   * @param params 请求参数
   * @param config 其他配置
   */
  get<T>(url: string, params?: object, config = {}): Promise<ResultData<T>> {
    return this.service.get(url, { params, ...config })
  }

  /**
   * post请求
   * @param url 请求地址
   * @param data 请求参数
   * @param config 其他配置
   */
  post<T>(url: string, data?: object, config = {}): Promise<ResultData<T>> {
    return this.service.post(url, data, config)
  }

  /**
   * put请求
   * @param url 请求地址
   * @param data 请求参数
   * @param config 其他配置
   */
  put<T>(url: string, data?: object, config = {}): Promise<ResultData<T>> {
    return this.service.put(url, data, config)
  }

  /**
   * delete请求
   * @param url 请求地址
   * @param data 请求参数
   * @param config 其他配置
   */
  delete<T>(url: string, data?: object, config = {}): Promise<ResultData<T>> {
    return this.service.delete(url, { data, ...config })
  }

  /**
   * patch请求
   * @param url 请求地址
   * @param data 请求参数
   * @param config 其他配置
   */
  patch<T>(url: string, data?: object, config = {}): Promise<ResultData<T>> {
    return this.service.patch(url, data, config)
  }

  /**
   * head请求
   * @param url 请求地址
   * @param config 其他配置
   */
  head<T>(url: string, config = {}): Promise<ResultData<T>> {
    return this.service.head(url, config)
  }

  /**
   * options请求
   * @param url 请求地址
   * @param config 其他配置
   */
  options<T>(url: string, config = {}): Promise<ResultData<T>> {
    return this.service.options(url, config)
  }

  /**
   * request请求
   * @param config 请求配置
   */
  request<T>(config: AxiosRequestConfig): Promise<ResultData<T>> {
    return this.service.request(config)
  }

  /**
   * 下载文件
   * @param url 请求地址
   * @param data 请求参数
   * @param config 其他配置
   */
  download(url: string, data?: object, config = {}): Promise<BlobPart> {
    return this.service.post(url, data, { ...config, responseType: 'blob' })
  }
}

export default new RequestHttp(config)
