import type { Options } from '@/components/Render/type'

export type NodeType =
  | 'start'
  | 'approval'
  | 'cc'
  | 'jump'
  | 'exclusive'
  | 'inclusive'
  | 'concurrent'
  | 'parallel'
  | 'timer'
  | 'notify'
  | 'condition'
  | 'end'

export interface InjectField {
  type: 'string' | 'expression'
  name: string
  value: string
}

export interface FlowNode {
  id: string
  pid?: string
  name: string
  type: NodeType
  executionListeners?: NodeListener[]
  next?: FlowNode
}

export interface NodeListener {
  event: string
  implementationType: 'class' | 'expression' | 'delegateExpression'
  implementation: string
}

export interface StartNode extends FlowNode {
  formProperties: FormProperty[]
}

export interface AssigneeNode extends FlowNode {
  // 审批方式
  assigneeType:
    | 'user'
    | 'role'
    | 'organization'
    | 'formOrganization'
    | 'choice'
    | 'self'
    | 'leader'
    | 'orgLeader'
    | 'formUser'
    | 'formRole'
    | 'autoRefuse'
  // 审批人
  users: string[]
  // 审批角色
  roles: string[]
  // 审批组织
  organizations: string[]
  // 表单内组织
  formOrganization: string
  // 表单内人员
  formUser: string
  // 表单内角色
  formRole: string
  // 主管
  leader: number
  // 组织主管
  orgLeader: number
  // 自选：true-多选，false-单选
  choice: boolean
  // 发起人自己
  self: boolean
}

export interface CcNode extends AssigneeNode {
  formProperties: FormProperty[]
}

export interface NotifyNode extends AssigneeNode {
  types: ('site' | 'email' | 'sms' | 'wechat' | 'dingtalk' | 'feishu')[]
  subject: string
  content: string
}

export interface ApprovalNode extends AssigneeNode {
  // 多人审批方式
  multi: 'sequential' | 'joint' | 'single'
  // 审批人为空时处理方式：reject-拒绝，pass-通过，admin-管理员，assign-指定人员
  nobody: 'refuse' | 'pass' | 'admin' | 'assign'
  // 审批人与发起人是同一人：self-由提交人自己审批、skip:自动跳过、leader:直属上级审批、orgLeader:部门负责人审批
  same: 'self' | 'skip' | 'leader' | 'orgLeader'
  // 多人审批通过比例
  multiPercent: number
  // 审批人为空时,指定人员
  nobodyUsers: string[]
  // 审批超时
  timeout: ApprovalTimeout
  // 表单字段
  formProperties: FormProperty[]
  // 操作权限
  operations: OperationPermissions
  // 任务监听器
  taskListeners?: NodeListener[]
}

export interface TimerNode extends FlowNode {
  waitType: 'duration' | 'date'
  unit: 'PT%sS' | 'PT%sM' | 'PT%sH' | 'P%sD' | 'P%sW' | 'P%sM'
  duration: number
  timeDate?: string
}

export interface JumpNode extends FlowNode {
  targetNode?: string
}

export interface HttpNode extends FlowNode {
  url?: string
  method?: 'GET' | 'POST' | 'PUT' | 'DELETE'
  headers?: Options[]
  body?: Options[]
  requestTimeout?: number
  errorUsers: string[]
}

export interface ServiceNode extends FlowNode {
  implementationType: string
  implementation: string
  fields: InjectField[]
}

export interface EndNode extends FlowNode {}

export interface ConditionNode extends FlowNode {
  def: boolean
  conditions: any
}

export interface BranchNode extends FlowNode {
  branches: FlowNode[]
}

export interface ExclusiveNode extends BranchNode {
  branches: ConditionNode[]
}

export interface InclusiveNode extends BranchNode {
  branches: ConditionNode[]
}

export interface ParallelNode extends BranchNode {}

export interface ErrorInfo {
  id: string
  name: string
  message: string
}

export interface FormProperty {
  // 字段ID
  id: string
  // 字段名称
  name: string
  // 只读
  readonly: boolean
  // 必填
  required: boolean
  // 隐藏
  hidden: boolean
}

export interface OperationPermissions {
  // 同意
  complete: boolean
  // 拒绝
  refuse: boolean
  // 回退
  back: boolean
  // 转交
  transfer: boolean
  // 委派
  delegate: boolean
  // 加签
  addMulti: boolean
  // 减签
  minusMulti: boolean
}

export interface ApprovalTimeout {
  enabled: boolean
  duration: number
  unit: 'PT%sS' | 'PT%sM' | 'PT%sH' | 'P%sD' | 'P%sW' | 'P%sM'
  action: 'pass' | 'refuse' | 'remind'
  loop: boolean
}
