import Render from '@/components/Render/index'
import type { PropType, VNode } from 'vue'
import { renderSlot } from 'vue'
import { cloneDeep } from 'lodash-es'
import type { FormConf, FormField } from './type'
import useForm from './useForm'
import type { FieldRule, FormInstance } from 'vant'

export default defineComponent({
  props: {
    formConf: {
      type: Object as PropType<FormConf>,
      required: true
    },
    formData: {
      type: Object as PropType<Recordable>,
      required: true
    }
  },
  emits: ['submit', 'reset'],
  setup(props, { slots, emit }) {
    const formConfCopy: FormConf = reactive(cloneDeep(props.formConf))
    const formRef = ref<FormInstance>()
    const { addObserver, delObserver, trigger } = useForm(formConfCopy, props.formData)

    /**
     * 布局
     */
    const layouts: Recordable<(field: FormField) => VNode> = {
      /**
       * 表单项
       * @param field
       */
      formItem: (field: FormField): VNode => {
        const prop = field.id
        const hidden = field.hidden
        field.props.rules = buildRule(field)
        field.props._vModel_ = prop
        field.props.readonly = field.readonly
        if (hidden) {
          return <div></div>
        }
        const child = renderChildren(field)
        return (
          <Render
            field={field}
            key={field.props.key}
            onAddObserver={addObserver}
            onDelObserver={delObserver}
            modelValue={props.formData[field.id]}
            onUpdate:modelValue={(val) => {
              if (field.readonly) return
              setValue(field, val)
            }}
          >
            {child}
          </Render>
        )
      },
      /**
       * 容器
       * @param field
       */
      container: (field: FormField): VNode => {
        const hidden = field.hidden
        if (hidden) {
          return <div></div>
        }
        const child = renderChildren(field)
        return (
          <Render
            field={field}
            key={field.props.key}
            modelValue={field.value}
            onUpdate:modelValue={(val) => {
              field.value = val
            }}
            onAddObserver={addObserver}
            onDelObserver={delObserver}
          >
            {child}
          </Render>
        )
      }
    }

    /**
     * 设置值
     * @param field
     * @param val
     */
    const setValue = (field: FormField, val: unknown) => {
      field.value && (field.value = val)
      if (field.id) {
        props.formData[field.id] = val
      }
      trigger(field.id)
    }
    /**
     * 渲染子组件
     * @param element
     */
    const renderChildren = (element: FormField): VNode[] | undefined => {
      const children: FormField[] | undefined = element.children
      if (!Array.isArray(children)) return
      return renderFormItem(children)
    }
    /**
     * 构建校验
     * @param field
     */
    const buildRule = (field: FormField) => {
      const { props, rules } = field
      let ruleList: FieldRule[] = []
      if (Array.isArray(rules)) {
        ruleList.push(
          ...rules.map((rule) => {
            return {
              pattern: rule.pattern && new RegExp(rule.pattern),
              message: `请输入正确的${rule.message}`
            }
          })
        )
      }
      if (field.required !== undefined) {
        const required: FieldRule = {
          required: field.required || false,
          message: props.placeholder
        }
        if (Array.isArray(field.value)) {
          required.message = `请至少选择一个${field.label}`
          required.trigger = 'onChange'
        }
        required.message === undefined && (required.message = `${field.label}不能为空`)
        ruleList.push(required)
      }
      return ruleList
    }
    /**
     * 渲染表单
     */
    const renderFrom = (): VNode => {
      return (
        <van-form
          label-align={'top'}
          disabled={formConfCopy.disabled}
          label-width={`${formConfCopy.labelWidth}px`}
          ref={formRef}
          onSubmit={submitForm}
        >
          {renderFormItem(formConfCopy.fields)}
          {renderSlot(slots, 'default', {
            fields: formConfCopy.fields,
            formRef: formRef.value,
            formData: props.formData
          })}
        </van-form>
      )
    }
    /**
     * 渲染表单项
     */
    const renderFormItem = (elementList: FormField[]): VNode[] => {
      return elementList.map((element: FormField) => {
        const layout = layouts[element.name] || layouts[element.type]
        if (layout) {
          return layout(element)
        }
        throw new Error(`未找到${element?.type}类型的布局`)
      })
    }
    /**
     * 提交表单
     */
    const submitForm = () => {
      return new Promise<Recordable>((resolve, reject) => {
        if (!formRef.value) {
          reject()
          return
        }
        formRef.value
          ?.validate()
          .then(() => {
            emit('submit', props.formData)
            resolve(props.formData)
          })
          .catch(() => {
            reject()
          })
      })
    }
    /**
     * 重置表单
     */
    const resetForm = () => {
      if (!formRef.value) return
      Object.keys(props.formData).forEach((key) => {
        trigger(key)
      })
      emit('reset', props.formData)
    }
    return {
      renderFrom,
      submitForm,
      resetForm
    }
  },
  render() {
    return this.renderFrom()
  }
})
