import type { Field, Options } from './type'
import type { PropType } from 'vue'
import { mergeProps } from 'vue'
import { cloneDeep } from 'lodash-es'
import asyncComponents from './components'
import componentChild from './slots'
import hooks from './hooks'
import type { Callback, FormField } from '@/components/FormParser/type'

export default defineComponent({
  props: {
    modelValue: {
      type: [String, Number, Boolean, Array, Object],
      required: false
    },
    field: {
      type: Object as PropType<FormField>,
      required: true
    }
  },
  emits: [
    'update:modelValue',
    'input',
    'change',
    'addObserver',
    'delObserver',
    'mounted',
    'unmount'
  ],
  components: {
    ...asyncComponents
  },
  setup(props, { emit }) {
    const callback: Callback = (options: Options[]) => {
      props.field.props.options = options
    }
    onMounted(() => {
      emit('mounted')
      const { dataType, dataSource } = props.field.props
      if (dataType === 'dynamic' && dataSource) {
        emit('addObserver', callback)
      }
    })
    onBeforeUnmount(() => {
      emit('unmount')
      const { dataType, dataSource } = props.field.props
      if (dataType === 'dynamic' && dataSource) {
        emit('delObserver', callback)
      }
    })
    const buildDataObject = (fieldClone: FormField) => {
      // 赋值属性
      const { props: nestedProps, ...rest } = fieldClone
      const dataObject: Recordable = { ...rest, ...nestedProps }
      /*dataObject.id = fieldClone.name
      dataObject.name = fieldClone.id*/
      // 双向绑定
      if (props.modelValue !== undefined) {
        dataObject.modelValue = props.modelValue
        dataObject['onUpdate:modelValue'] = (value: any) => {
          emit('update:modelValue', value)
          emit('input', value)
          emit('change', value)
        }
      }
      // 删除无用属性
      delete dataObject.options
      delete dataObject.dataType
      delete dataObject.dataSource
      return dataObject
    }
    const buildSlots = (fieldClone: Field) => {
      let slots: Recordable<(...args: any) => void> = {}
      const childObjs = componentChild[fieldClone.name]
      if (childObjs) {
        Object.keys(childObjs).forEach((key) => {
          const childFunc = childObjs[key]
          slots[key] = (...args: any) => childFunc(fieldClone, ...args)
        })
      }
      return slots
    }
    const useHooks = (fieldClone: Field) => {
      const childObjs = hooks[fieldClone.name]
      if (childObjs) {
        Object.keys(childObjs).forEach((key) => {
          const mixinFunc = childObjs[key]
          mixinFunc.call(this, fieldClone)
        })
      }
    }
    return {
      buildDataObject,
      useHooks,
      buildSlots
    }
  },
  render() {
    const fieldClone: FormField = cloneDeep(this.field)
    const slots = this.buildSlots(fieldClone)
    this.useHooks(fieldClone)
    const dataObject = this.buildDataObject(fieldClone)
    const componentName = fieldClone.props.componentName || fieldClone.name
    const eleComponent = resolveComponent(componentName)
    return h(eleComponent, mergeProps(dataObject, this.$attrs), { ...this.$slots, ...slots })
  }
})
